summaryrefslogtreecommitdiffhomepage
path: root/examples/core
diff options
context:
space:
mode:
authorRay <[email protected]>2019-04-11 13:53:01 +0200
committerRay <[email protected]>2019-04-11 13:53:01 +0200
commit5bfa67535040ed1cb717908d9f259db0c302d9e6 (patch)
tree5d32732a4f76382b4098cc8cd4226b2aaba934b9 /examples/core
parentaa8d10c48aed2bd41118156aead6eda26456b34f (diff)
downloadraylib-5bfa67535040ed1cb717908d9f259db0c302d9e6.tar.gz
raylib-5bfa67535040ed1cb717908d9f259db0c302d9e6.zip
Review VR simulator mechanism
- No default VR device parameteres inside raylib - VR device parameter should be provided by user - VR distortion shader should be provided by user
Diffstat (limited to 'examples/core')
-rw-r--r--examples/core/core_vr_simulator.c38
-rw-r--r--examples/core/resources/distortion.fs43
2 files changed, 77 insertions, 4 deletions
diff --git a/examples/core/resources/distortion.fs b/examples/core/resources/distortion.fs
new file mode 100644
index 00000000..207a0aff
--- /dev/null
+++ b/examples/core/resources/distortion.fs
@@ -0,0 +1,43 @@
+#version 330
+
+in vec2 fragTexCoord;
+in vec4 fragColor;
+out vec4 finalColor;
+
+uniform sampler2D texture0;
+uniform vec2 leftLensCenter = vec2(0.288, 0.5);
+uniform vec2 rightLensCenter = vec2(0.712, 0.5);
+uniform vec2 leftScreenCenter = vec2(0.25, 0.5);
+uniform vec2 rightScreenCenter = vec2(0.75, 0.5);
+uniform vec2 scale = vec2(0.25, 0.45);
+uniform vec2 scaleIn = vec2(4, 2.2222);
+uniform vec4 hmdWarpParam = vec4(1, 0.22, 0.24, 0);
+uniform vec4 chromaAbParam = vec4(0.996, -0.004, 1.014, 0.0);
+
+void main()
+{
+ vec2 lensCenter = fragTexCoord.x < 0.5? leftLensCenter : rightLensCenter;
+ vec2 screenCenter = fragTexCoord.x < 0.5? leftScreenCenter : rightScreenCenter;
+ vec2 theta = (fragTexCoord - lensCenter)*scaleIn;
+ float rSq = theta.x*theta.x + theta.y*theta.y;
+ vec2 theta1 = theta*(hmdWarpParam.x + hmdWarpParam.y*rSq + hmdWarpParam.z*rSq*rSq + hmdWarpParam.w*rSq*rSq*rSq);
+ vec2 thetaBlue = theta1*(chromaAbParam.z + chromaAbParam.w*rSq);
+ vec2 tcBlue = lensCenter + scale*thetaBlue;
+
+ if (any(bvec2(clamp(tcBlue, screenCenter - vec2(0.25, 0.5), screenCenter + vec2(0.25, 0.5)) - tcBlue)))
+ {
+ finalColor = vec4(0.0, 0.0, 0.0, 1.0);
+ }
+ else
+ {
+ float blue = texture(texture0, tcBlue).b;
+ vec2 tcGreen = lensCenter + scale*theta1;
+ float green = texture(texture0, tcGreen).g;
+
+ vec2 thetaRed = theta1*(chromaAbParam.x + chromaAbParam.y*rSq);
+ vec2 tcRed = lensCenter + scale*thetaRed;
+
+ float red = texture(texture0, tcRed).r;
+ finalColor = vec4(red, green, blue, 1.0);
+ }
+};
diff --git a/examples/core/core_vr_simulator.c b/examples/core/core_vr_simulator.c
index 3f59e839..65054b73 100644
--- a/examples/core/core_vr_simulator.c
+++ b/examples/core/core_vr_simulator.c
@@ -23,7 +23,35 @@ int main()
InitWindow(screenWidth, screenHeight, "raylib [core] example - vr simulator");
// Init VR simulator (Oculus Rift CV1 parameters)
- InitVrSimulator(GetVrDeviceInfo(HMD_OCULUS_RIFT_CV1));
+ InitVrSimulator();
+
+ VrDeviceInfo hmd = { 0 }; // VR device parameters (head-mounted-device)
+
+ // Oculus Rift CV1 parameters for simulator
+ hmd.hResolution = 2160; // HMD horizontal resolution in pixels
+ hmd.vResolution = 1200; // HMD vertical resolution in pixels
+ hmd.hScreenSize = 0.133793f; // HMD horizontal size in meters
+ hmd.vScreenSize = 0.0669f; // HMD vertical size in meters
+ hmd.vScreenCenter = 0.04678f; // HMD screen center in meters
+ hmd.eyeToScreenDistance = 0.041f; // HMD distance between eye and display in meters
+ hmd.lensSeparationDistance = 0.07f; // HMD lens separation distance in meters
+ hmd.interpupillaryDistance = 0.07f; // HMD IPD (distance between pupils) in meters
+
+ // NOTE: CV1 uses a Fresnel-hybrid-asymmetric lenses with specific distortion compute shaders.
+ // Following parameters are an approximation to distortion stereo rendering but results differ from actual device.
+ hmd.lensDistortionValues[0] = 1.0f; // HMD lens distortion constant parameter 0
+ hmd.lensDistortionValues[1] = 0.22f; // HMD lens distortion constant parameter 1
+ hmd.lensDistortionValues[2] = 0.24f; // HMD lens distortion constant parameter 2
+ hmd.lensDistortionValues[3] = 0.0f; // HMD lens distortion constant parameter 3
+ hmd.chromaAbCorrection[0] = 0.996f; // HMD chromatic aberration correction parameter 0
+ hmd.chromaAbCorrection[1] = -0.004f; // HMD chromatic aberration correction parameter 1
+ hmd.chromaAbCorrection[2] = 1.014f; // HMD chromatic aberration correction parameter 2
+ hmd.chromaAbCorrection[3] = 0.0f; // HMD chromatic aberration correction parameter 3
+
+ // Distortion shader (uses device lens distortion and chroma)
+ Shader distortion = LoadShader(0, "resources/distortion.fs");
+
+ SetVrConfiguration(hmd, distortion); // Set Vr device parameters for stereo rendering
// Define the camera to look into our 3d world
Camera camera;
@@ -77,9 +105,11 @@ int main()
// De-Initialization
//--------------------------------------------------------------------------------------
- CloseVrSimulator(); // Close VR simulator
-
- CloseWindow(); // Close window and OpenGL context
+ UnloadShader(distortion); // Unload distortion shader
+
+ CloseVrSimulator(); // Close VR simulator
+
+ CloseWindow(); // Close window and OpenGL context
//--------------------------------------------------------------------------------------
return 0;